#!/usr/bin/env python3

from datetime import datetime

import numpy as np
import pylab

from numpy import pi, square, log10

from util_noise4 import TransferFunction, abs_squared, get_cov, sleeman, holcomb


class DataSource( object ):

    def __init__( self, network, station, location, sensor, theta ):

        self.params = {}
        self.params['network'] = network
        self.params['station'] = station
        self.params['location'] = location
        self.params['sensor'] = sensor
        self.params['theta'] = theta


    def get_filename( self, time, something):

        filename_date = datetime.strftime(time, "%Y_%%s_%H_%M")
        filename_date %= datetime.strftime(time, "%j").lstrip("0")

        return "DATA_{network}_{station}_{location}_{}_{}.bdf".format(
            something, filename_date, **self.params
            )


    def load_data( self, time, something ):

        filename = self.get_filename(time, something)

        print("loading `{}'...".format(filename))

        with open(filename, r'r') as fp:

            # Skip two first lines
            fp.readline()
            fp.readline()

            raw_data = fp.readlines()

        return list(map(np.float16, raw_data))


class TestDataSource( DataSource ):

    def __init__( self, filename, **kwargs ):

        self.params = {}
        self.params['filename'] = filename
        self.params.update(kwargs)


    def get_filename( self, *params, **kwargs ):

        return self.params['filename']

sources = [
    TestDataSource('Data1.bdf', sensor = 'T120'),
    TestDataSource('Data2.bdf', sensor = 'KS1a'),
    TestDataSource('Data3.bdf', sensor = 'CMG3'),
#    DataSource(network = 'XX', station = 'FBA1', location = '00', sensor = 'T120', theta = 0),
#    DataSource(network = 'XX', station = 'ANMO', location = '00', sensor = 'KS1a', theta = 0),
#    DataSource(network = 'IU', station = 'ANMO', location = '10', sensor = 'CMG3', theta = 0)
]

transfer_function = {
    'T120' : TransferFunction.fromstream(open("SAC_PZs_Data1")),
    'KS1a' : TransferFunction.fromstream(open("SAC_PZs_Data2")),
    'CMG3' : TransferFunction.fromstream(open("SAC_PZs_Data3")),
}


def plot_nlnm():

    nlnm = np.loadtxt("NLNM.txt", comments = '=', unpack = True)

    nlnm_freq = 1/nlnm[0] 
    nlnm_amp = nlnm[3]

    pylab.plot(nlnm_freq, nlnm_amp, '-', label = "NLNM")

    return nlnm_freq


def main():

    SAMPLE_RATE_BHZ = 40
    SAMPLE_RATE_LHZ = 1*2*2*2

    channels_data = [src.load_data(datetime.now(), "BHZ")
                     for src in sources]

    (cov_mat, freqs) = get_cov(channels_data, SAMPLE_RATE_BHZ)

    # we can remove first element since it will result in NaN anyway
    cov_mat = cov_mat.take(range(1, cov_mat.shape[2]), 2)
    freqs = freqs[1:]

    tf_values = []

    for src in sources:

        sensor = src.params['sensor']
        tf_values.append(transfer_function[sensor].get_value(freqs))

    noise_sleeman = sleeman(cov_mat, freqs)
   # noise_holcomb = holcomb(freqs, cov_mat, tf_values)

    color = ['r', 'g', 'b']

    for i, values in enumerate(noise_sleeman):

        ys = 10 * log10(values / abs_squared(tf_values[i]))

        pylab.plot(freqs, ys, '-', label = "Noise of Data {}".format(i+1), color = color[i])


   # for i, values in enumerate(noise_holcomb):

   #     ys = 10 * log10(abs(values))

   #     pylab.plot(freqs, ys, '.', label = "Holcomb noise of Data {}".format(i+1), color = color[i])


    for i in range(len(sources)):

        ys = 10 * log10(
            square(2 * pi * freqs) * cov_mat[i][i] / abs_squared(tf_values[i])
            )

        pylab.plot(freqs, ys, '--', label = "PSD of Data {}".format(i+1))

    nlnm_freq = plot_nlnm()

    pylab.xlim(min(nlnm_freq), max(nlnm_freq))
    pylab.xscale('log')
    pylab.legend(loc = 2)
    pylab.title("Sleeman Self Noise")
    pylab.grid()
    pylab.show()


if __name__ == '__main__':

    main()
